package Logoved::Role::Stream::Out::Pipe;
use strict;
use warnings;
use Moo::Role;
with 'Logoved::Role::Stream::Out::Source';

requires 'accept';

has count => (is => 'rw', default => 0);

has listeners_factory => (is => 'ro');

has listeners => (is => 'lazy');

has section_type => (is => 'lazy');

has section_id => (is => 'lazy');

sub _build_section_id {
    my ($self) = @_;
    return &static_method_section_type($_[0]);
}

sub static_method_section_type {
    my ($class) = @_;
    $class=ref($class) if ref($class);
    #$class=~s/^Logoved::Stream::Out::(?:Section::)?//;
    #return $class;
    if ($class=~s/^Logoved::Stream::Out::Section:://) {
	return $class;
    } else {
	# main container: Rpmbuild, Hasher
	return '';
    }
}

sub _build_section_type {
    return &static_method_section_type($_[0]);
}

sub _build_listeners {
    my ($self) = @_;
    my $factory=$self->listeners_factory;
    unless ($factory) {
	return [];
    } else {
	return $factory->get_listeners_for($self);
    }
}

after 'write_listeners' => sub {
    $_[0]->{'count'}++;
};

sub write_listeners {
    my ($self, $line) = @_;
    local $_;
    map {$_->write($line)} @{$self->listeners};
}

sub commit_listeners {
    my ($self) = @_;
    local $_;
    map {$_->commit()} @{$self->listeners};
}

sub error_listeners {
    my ($self) = @_;
    local $_;
    map {$_->error} @{$self->listeners};
}

sub success_listeners {
    my ($self) = @_;
    local $_;
    map {$_->success} @{$self->listeners};
}

1;

__END__
