package Logoved::Stream::In;
use Moo;
use autodie qw/open close/;
use File::Basename;

# count is from 0, line number is from 1
has 'lineno' => (is => 'rw', default => 0);
has 'file' => (is => 'ro');
has 'name' => (is => 'ro');
has 'fh' => (is => 'rw');
has 'buffer' => (is => 'ro', default => sub { [] } );
with 'Logoved::Role::Stream::In';

sub pushback {
    my $self=shift;
    my $buffer=$self->buffer;
    my $lineno=$self->{'lineno'};
    $self->{'lineno'}=$lineno - scalar @_;
    push @$buffer, @_;
}

sub pushback_chunk {
    my ($self, $chunk)=@_;
    my $buffer=$self->buffer;
    my $lineno=$self->{'lineno'};
    $self->{'lineno'}=$lineno - scalar @$chunk;
    push @$buffer, @$chunk;
}

sub read {
    my $self=shift;
    my $buffer=$self->buffer;
    $self->{'lineno'}++;
    return shift @$buffer if @$buffer;
    my $fh=$self->fh;
    my $line=<$fh>;
    return $line;
}

sub create_from_file {
    my $class=shift;
    my $input=shift;
    open(my $fh, '<', $input); #or die "can't open $input: $!";
    my $name=basename($input);
    # hack; autorepo logs have the same name, only parent dirs differ
    $name=basename(dirname($input)).'/'.$name if length($name) < 10 or substr($name,0,7) eq 'hsh.log';
    return $class->new(fh => $fh, name => $name, file => $input, @_);
}

sub create_from_filehandler {
    my $class=shift;
    my $fh=shift;
    return $class->new(fh => $fh, @_);
}

sub commit {
    my $self=shift;
    my $fh=$self->fh;
    close($fh) if $fh;
    $self->fh(undef);
}

sub DEMOLISH {
    my $self=shift;
    $self->commit;
}

1;

__END__
