package Logoved::Stream::Listener::Factory;
use Carp;
use Moo;
with 'Logoved::Role::Stream::Listener::Factory';

has 'global_args' => (is => 'ro', default => sub { {} } );
has 'listener_args' => (is => 'ro', default => sub { {} } );

sub get_listener_classes {}

sub configure {
    my $self=shift;
    my $argsref=$self->global_args;
    while (scalar @_ >1) {
	my $key=shift;
	my $val=shift;
	$argsref->{$key}=$val;
    }
    confess "Odd number of arguments" if @_;
}

sub configure_listener {
    my $self=shift;
    my $listener_class=shift;
    my $old_args=$self->listener_args->{$listener_class};
    $old_args//=[];
    my %args=(@$old_args, @_);
    $self->listener_args->{$listener_class}=[%args];
}

sub get_global_arg {
    my ($self, $key)=@_;
    return $self->global_args->{$key};
}

#sub _get_local_args {return;}

sub get_listener_args {
    my ($self, $class)=@_;
    croak "undefined class" unless defined $class;
    my @args=(listeners_factory => $self);
    push @args, (%{$self->global_args});
    my $add_ref=$self->listener_args->{$class};
    push @args, @$add_ref if $add_ref;
#    push @args, $self->_get_local_args();
    return @args;
}

# ---- interface implementation ---------------------

sub init_new_stream {}
sub done_new_stream {}

sub get_listeners_for {
    my ($self, $stream)=@_;
    my @listeners;
    foreach my $class ($self->get_listener_classes) {
	push @listeners, $class->new(
	    stream => $stream,
	    $self->get_listener_args($class),
	    ) if $class->subscribe_stream($stream);
    }
    return \@listeners;
}

1;
