package Logoved::Stream::Out::Section::hasher::chroot::install;
use Logoved::Stream::Out::Section::common::cleanup qw/cleanup_macros_artefacts/;
use Moo;
extends 'Logoved::Stream::Out::Section';
with 'Logoved::Role::Stream::Out::Section';

our $re_log = qr /^<\d+>\S+\s+\d+ [0-9:]+ /;
our $re_rpmi = qr /^<13>\S+\s+\d+ [0-9:]+ rpmi:/;

# sub is not mandatory, because some packages install nothing

sub get_error_classes {
    use Logoved::Stream::Out::Section::hasher::timeout;
    use Logoved::Stream::Out::Section::hasher::chroot::apt;
    use Logoved::Stream::Out::Section::hasher::chroot::file_conflict;
    use Logoved::Stream::Out::Section::hasher::chroot::error;
    use Logoved::Stream::Out::Section::hasher::chroot::sisyphus_check;
    return qw/
	Logoved::Stream::Out::Section::hasher::timeout
	Logoved::Stream::Out::Section::hasher::chroot::apt
	Logoved::Stream::Out::Section::hasher::chroot::file_conflict
	Logoved::Stream::Out::Section::hasher::chroot::error
	Logoved::Stream::Out::Section::hasher::chroot::sisyphus_check
	/;
}

sub match_section_start {
    my ($self, $line) = @_;
    $line=cleanup_macros_artefacts($line);
    return $line=~ $re_rpmi;
};

#use Logoved::Stream::Out::Section::hasher::chroot::srpm::warnings;

sub accept {
    my ($self, $line) = @_;
    # disabled - install is too complex

    #$line=cleanup_macros_artefacts($line);
    #return 0 if $line=~ $Logoved::Stream::Out::Section::hasher::chroot::srpm::warnings::re;

# accept all due to the %pre/%post messages
#Updating fonts cache: <29>Aug  1 12:00:01 fontconfig: Updating fonts cache: succeeded
#[ DONE ]
    #return $line=~ $re_log;
    return 1;
}

# not used: see accept (accepts all)
sub unshift_sections_on_reject {
    my $self=shift;
    return qw/Logoved::Stream::Out::Section::hasher::chroot::srpm::warnings/;
};


1;
__END__
buildrequires+install:
fetch-buildrequires-pre
install-buildrequires-pre
fetch-buildrequires
install-buildrequires
beehive-repackage-srpm
install srpm
-----------------------------
<13>Aug  7 00:54:27 rpmi: libboost_system1.67.0-1:1.67.0-alt4 1533085122 installed
<13>Aug  7 00:54:27 rpmi: libexpat-2.2.4-alt1 1503305341 installed
<13>Aug  7 00:54:27 rpmi: libopenblas-0.2.20-alt1 1527006103 installed
<13>Aug  7 00:54:30 rpmi: libcrypto10-1.0.2o-alt1 1522836697 installed
<86>Aug  7 00:54:30 groupadd[23346]: group added to /etc/group: name=_keytab, GID=499
<86>Aug  7 00:54:30 groupadd[23346]: group added to /etc/gshadow: name=_keytab
<86>Aug  7 00:54:30 groupadd[23346]: new group: name=_keytab, GID=499
<13>Aug  7 00:54:31 rpmi: libkrb5-1.16-alt1.S1 1516715375 installed
<13>Aug  7 00:54:31 rpmi: libssl10-1.0.2o-alt1 1522836697 installed
<13>Aug  7 07:00:07 rpmi: rpm-build-java-1:5.0.0-alt1_12jpp8 1525973129 installed
sh  JAVA_HOME is not set and default java installation was not found. JAVA_HOME for system applications can be set in java.conf in /etc/java
<13>Aug  7 07:00:11 rpmi: xorg-proto-devel-2018.4-alt3 1527685079 installed
<13>Aug  7 07:19:01 rpmi: fontconfig-2.12.6-alt1.S1 1506000395 installed
Updating fonts cache: [ DONE ]
<29>Aug  7 07:19:02 fontconfig: Updating fonts cache: succeeded
<13>Aug  7 07:19:02 rpmi: libharfbuzz-icu-1.8.5-alt1 1533195837 installed
