package Logoved::Stream::Out::Section::hasher::chroot::users;
use Moo;
extends 'Logoved::Stream::Out::Section';
with 'Logoved::Role::Stream::Out::Section';

sub is_mandatory {return 1};

sub get_error_classes {
    use Logoved::Stream::Out::Section::hasher::timeout;
    use Logoved::Stream::Out::Section::hasher::chroot::apt;
    use Logoved::Stream::Out::Section::hasher::chroot::file_conflict;
    use Logoved::Stream::Out::Section::hasher::chroot::error;
    use Logoved::Stream::Out::Section::hasher::chroot::sisyphus_check;
    return qw/
	Logoved::Stream::Out::Section::hasher::timeout
	Logoved::Stream::Out::Section::hasher::chroot::apt
	Logoved::Stream::Out::Section::hasher::chroot::file_conflict
	Logoved::Stream::Out::Section::hasher::chroot::error
	Logoved::Stream::Out::Section::hasher::chroot::sisyphus_check
	/;
}

sub match_section_start {
    my ($self, $line) = @_;
    return $line=~/^<86>\S+\s+\d+ [0-9:]+ (?:group|user)(?:add|del)\[\d+\]: .*[='](?:caller|rooter|builder)\b/;
};

sub accept {
    my ($self, $line) = @_;
    return $line=~/^<86>\S+\s+\d+ [0-9:]+ (?:group|user)(?:add|del)\[\d+\]: .*[='](?:caller|rooter|builder)\b/;
};

use Logoved::Stream::Out::Section::hasher::chroot::warning::sisyphus_check;
use Logoved::Stream::Out::Section::hasher::chroot::buildrequires;
use Logoved::Stream::Out::Section::hasher::chroot::install;

sub get_warning_classes {
    my $self=shift;
    return qw/
	Logoved::Stream::Out::Section::hasher::chroot::warning::sisyphus_check
	/;
};

sub unshift_sections_on_reject {
    my $self=shift;
    return qw/
	Logoved::Stream::Out::Section::hasher::chroot::buildrequires
	Logoved::Stream::Out::Section::hasher::chroot::install
	/;
};

1;
__END__
<86>Aug  1 12:05:40 userdel[16275]: delete user 'rooter'
<86>Aug  1 12:05:40 userdel[16275]: removed group 'rooter' owned by 'rooter'
<86>Aug  1 12:05:40 userdel[16275]: removed shadow group 'rooter' owned by 'rooter'
<86>Aug  1 12:05:40 groupadd[16290]: group added to /etc/group: name=rooter, GID=547
<86>Aug  1 12:05:40 groupadd[16290]: group added to /etc/gshadow: name=rooter
<86>Aug  1 12:05:40 groupadd[16290]: new group: name=rooter, GID=547
<86>Aug  1 12:05:40 useradd[16297]: new user: name=rooter, UID=547, GID=547, home=/root, shell=/bin/bash
<86>Aug  1 12:05:40 userdel[16316]: delete user 'builder'
<86>Aug  1 12:05:40 userdel[16316]: removed group 'builder' owned by 'builder'
<86>Aug  1 12:05:40 groupadd[16325]: group added to /etc/group: name=builder, GID=548
<86>Aug  1 12:05:40 groupadd[16325]: group added to /etc/gshadow: name=builder
<86>Aug  1 12:05:40 groupadd[16325]: new group: name=builder, GID=548
<86>Aug  1 12:05:40 useradd[16338]: new user: name=builder, UID=548, GID=548, home=/usr/src, shell=/bin/bash
Reading Package Lists...
Building Dependency Tree...
E: Couldn't find package qtsingleapplication-qt5-devel
hsh-install: Failed to calculate package file list.
hsh-install: Failed to generate package file list.
---------------------------------------------------
<13>Aug  1 11:59:34 rpmi: setup-2.2.15-alt1 1503594490 installed
<13>Aug  1 11:59:34 rpmi: filesystem-2.3.16-alt1 1515521139 installed
<13>Aug  1 11:59:34 rpmi: glibc-preinstall-6:2.27-alt6 1530126039 installed
[...]
<13>Aug  1 11:59:44 rpmi: time-1:1.7-alt4 1449722762 installed
<86>Aug  1 11:59:45 groupadd[15254]: group added to /etc/group: name=caller, GID=544
<86>Aug  1 11:59:45 groupadd[15254]: group added to /etc/gshadow: name=caller
<86>Aug  1 11:59:45 groupadd[15254]: new group: name=caller, GID=544
<86>Aug  1 11:59:45 useradd[15269]: new user: name=caller, UID=544, GID=544, home=/, shell=/bin/bash
<86>Aug  1 11:59:45 groupadd[15306]: group added to /etc/group: name=rooter, GID=547
<86>Aug  1 11:59:45 groupadd[15306]: group added to /etc/gshadow: name=rooter
<86>Aug  1 11:59:45 groupadd[15306]: new group: name=rooter, GID=547
<86>Aug  1 11:59:45 useradd[15324]: new user: name=rooter, UID=547, GID=547, home=/root, shell=/bin/bash
<86>Aug  1 11:59:45 groupadd[15355]: group added to /etc/group: name=builder, GID=548
<86>Aug  1 11:59:45 groupadd[15355]: group added to /etc/gshadow: name=builder
<86>Aug  1 11:59:45 groupadd[15355]: new group: name=builder, GID=548
<86>Aug  1 11:59:45 useradd[15377]: new user: name=builder, UID=548, GID=548, home=/usr/src, shell=/bin/bash
