package Logoved::Stream::Out::Switch;
use Carp;
use Moo;

has 'is_switched_to_error' => (is => 'rw');
has 'active_section' => (is => 'rw',);
# pop_up section pop up and put current section to background. When pop_up section no more accept,
# control is returned to the background section
has 'pop_up_sections' => (
    is => 'rw',
    default => sub { [] },
    );
has 'background_section' => (
    is => 'rw',
    );
# warning sections are pop-up sections that has priority over error sections
has 'warning_sections' => (
    is => 'rw',
    default => sub { [] },
    );
has 'error_sections' => (
    is => 'rw',
    default => sub { [] },
    );
has 'sectionname_listref' => (
    is => 'lazy',
    isa => sub { die $_[0],': section_list expected an array ref' unless ref($_[0]) eq 'ARRAY'},
    );
has 'sections' => (
    is => 'lazy',
    );
# events
has 'sectionname_for_push_on_active_section_switch' => (
    is => 'lazy',
    );
with 'Logoved::Role::Stream::Out::Pipe';
my $debug=0;
#use Debuggit DEBUG => 2;

sub __section_args {
    my ($self)=@_;
    #warn 'NAME=', $self->name,"\n";
    return
#	name => $self->name,
	listeners_factory => $self->listeners_factory,
	messager => $self->messager,
	source => $self->source,
	;
}

sub _build_sectionname_listref {
    return [];
}

sub _build_sectionname_for_push_on_active_section_switch {
    return {};
}

sub _build_sections {
    my ($self)=@_;
    my @sections=$self->_section_factory(@{$self->sectionname_listref});
    #warn "called _build_sections: got ",join(',',map {$_->section_type} @sections),"\n";
    return \@sections;
}

sub _section_factory {
    return if @_<2;
    my $self=shift;
    my @args=$self->__section_args();
    local $_;
    return map {$_->new(@args)} @_;
}

sub __change_active_section {
    my ($self, $new_section) = @_;
    my $old_section=$self->active_section;
    &__commit_section($old_section, $self->is_switched_to_error) if $old_section;
    $self->active_section($new_section);
    my $new_class=ref($new_section);
    my $push_event_hash=$self->sectionname_for_push_on_active_section_switch;
    my $push_sections_ref=$push_event_hash->{$new_class};
    if ($push_sections_ref and scalar @$push_sections_ref) {
	my $sec_ref=$self->sections;
	push @$sec_ref, $self->_section_factory(@$push_sections_ref);
	delete $push_event_hash->{$new_class};
	warn 'DEBUG: ',$self->name,": pushed on change ", join(' ',@$push_sections_ref) , "\n" if $debug>1;
	warn 'DEBUG: ',$self->name,": pushed to ", join(' ',@$sec_ref) , "\n" if $debug>2;
    }
    if (!$new_section->keep_error_classes()) {
	my @error_classes=$new_section->get_error_classes();
	$self->error_sections([$self->_section_factory(@error_classes)]);
    }
    if (!$new_section->keep_warning_classes()) {
	my @warning_classes=$new_section->get_warning_classes();
	$self->warning_sections([$self->_section_factory(@warning_classes)]);
    }
    if (!$new_section->keep_pop_up_classes()) {
	my @pop_up_classes=$new_section->get_pop_up_classes();
	$self->pop_up_sections([$self->_section_factory(@pop_up_classes)]);
    }
    if ($debug>1) {
	warn 'DEBUG: ',$self->section_type,' changed active section to ',$new_section->section_type,"\n";
	warn 'DEBUG: ',$self->section_type,' set error sections to ',join(',', map {$_->section_type} @{$self->error_sections}),"\n";
	warn 'DEBUG: ',$self->section_type,' set warning sections to ',join(',', map {$_->section_type} @{$self->warning_sections}),"\n";
	warn 'DEBUG: ',$self->section_type,' set pop_up sections to ',join(',', map {$_->section_type} @{$self->pop_up_sections}),"\n";
	if ($debug>2) {
	    my @sec_types=map {$_->section_type} @{$self->sections};
	    warn 'DEBUG: ',$self->name,": ",$self->section_type,' sections:', ( scalar @sec_types ? "\n\t".join("\n\t", @sec_types) : ' <none>' ), "\n";
	}
    }
    return $new_section;
}

sub write {
    my ($self, $line) = @_;
    $self->write_listeners($line);
    $self->write_switch_sections($line);
}

sub write_switch_sections {
    my ($self, $line) = @_;
    my $active_section=$self->active_section;
    confess "ERROR: internal: check if the line is accepted first: $line" unless $active_section;
    $active_section->write($line);
}

sub try_error_section {
    my ($self, $line) = @_;
    my $active_section=$self->active_section;
    my $error_sections=$self->error_sections;
    if ($active_section and $error_sections and @$error_sections) {
      MATCH1:
	foreach my $error_section (@$error_sections) {
	    if ($error_section->match_section_start($line)) {
		# clean up stack of expected sections
		my $sec_ref=$self->sections;
		@$sec_ref=();
		$self->is_switched_to_error(1);
		$self->__change_active_section($error_section);
		warn 'DEBUG: ',$self->name,": match error $active_section: $line" if $debug;
		return 1;
	    }
        }
    }
    return 0;
}

sub try_warning_section {
    my ($self, $line) = @_;
    return $self->__try_pop_up_section_with($line, $self->warning_sections);
}

sub try_pop_up_section {
    my ($self, $line) = @_;
    return $self->__try_pop_up_section_with($line, $self->pop_up_sections);
}

sub __try_pop_up_section_with {
    my ($self, $line, $pop_up_sectionref) = @_;
    my $active_section=$self->active_section;
    if ($active_section and $pop_up_sectionref and @$pop_up_sectionref) {
	foreach my $pop_up_section (@$pop_up_sectionref) {
	    if ($pop_up_section->match_section_start($line)) {
		# push_active_section_to_background
		$self->background_section($active_section);
		$self->__change_active_section($pop_up_section);
		warn 'DEBUG: ',$self->name,": match pop up $active_section: $line" if $debug;
		return 1;
	    }
        }
    }
    return 0;
}


sub switch_active_section {
    my ($self, $line) = @_;
    if ($self->try_warning_section($line) or
	$self->try_error_section($line) or
	$self->try_pop_up_section($line)
	) {
	return $self->active_section;
    }
    my $sec_ref=$self->sections;
    my $sec_count=scalar @$sec_ref;
    MATCH2:
    for (my $i=0; $i<$sec_count; $i++) {
	if ($sec_ref->[$i]->match_section_start($line)) {
	    my $new_section=$sec_ref->[$i];
	    warn 'DEBUG: ',$self->name,": match $i ", $new_section->section_type, ": $line" if $debug;
	    if ($i>0) {
		for (my $j=0; $j<$i; $j++) {
		    my $skipsec=$sec_ref->[$j];
		    warn 'WARN: ',$self->name,": skipped $skipsec for $new_section: $line" if $skipsec->is_mandatory;
		    $self->error() if $skipsec->is_required_for_success;
		}
	    }
	    @$sec_ref=@$sec_ref[$i+1 .. $sec_count-1];
	    $self->__change_active_section($new_section);
	    last MATCH2;
	#} else {
	#    warn 'DEBUG: ',$self->name,": don't match $i ", $sec_ref->[$i]->section_type, ": $line" if $debug>3;
	}
    }
    my $active_section=$self->active_section;
    if (! $active_section) {
	warn __PACKAGE__, ': ',$self->name,': line match nothing: ',$line;
	return;
    }
    return $active_section;
}

sub accept {
    my ($self, $line) = @_;
    #warn 'DEBUG: entered accept(switch) for ', $line, "\n" if $debug>5;
    # pop up sections support
    my $background_section=$self->background_section;
    if ($background_section) {
	my $active_section=$self->active_section;
	if ($active_section->accept($line)) {
	    return 1;
	} else {
	    #&__commit_section($active_section, 0);
	    &__commit_section($active_section, $self->is_switched_to_error);
	    $self->active_section($background_section);
	    $self->background_section(undef);
	}
    }
    # end pop up sections support
    my $active_section=$self->switch_active_section($line);
    return unless $active_section;
    my $sec_ref=$self->sections;
    while (not $active_section->accept($line)) {
	&__commit_section($active_section, $self->is_switched_to_error);
	$self->active_section(undef);
	my @inject_sections=$active_section->unshift_sections_on_reject();
	if (scalar @inject_sections) {
	    unshift @$sec_ref, $self->_section_factory(@inject_sections);
	    warn 'DEBUG: ',$self->name,": unshifted on reject: ", join(' ',@inject_sections) , "\n" if $debug>1;
	    #warn 'DEBUG: ',$self->name,": unshifted to ", join(' ',@$sec_ref) , "\n" if $debug>2;
	} else {
	    warn 'WARN: ',$self->name,': ',$active_section->section_type,": rejected $line";
	}
	$active_section=$self->switch_active_section($line);
	return unless $active_section;
    }
    return 1;
};

sub commit {
    my $self = shift;
    if ($self->is_switched_to_error) {
	$self->error();
    } else {
	$self->success();
    }
    $self->commit_listeners();
    $self->commit_switch_sections();
};

sub commit_switch_sections {
    my $self = shift;
    my $active_section=$self->active_section;
    #warn 'commit_switch_sections(',$self->section_type,'): active:',($active_section?$active_section->section_type:'<NULL>'),"\n";
    return unless $active_section;
    &__commit_section($active_section, $self->is_switched_to_error);
};

sub __commit_section {
    my ($section, $is_switched_to_error)=@_;
    if ($is_switched_to_error) {
	$section->error();
    } else {
	$section->success();
    }
    $section->commit();
}

sub success {
    my $self = shift;
    $self->success_listeners();
    $self->success_switch_sections();
}

sub error {
    my $self = shift;
    $self->error_listeners();
    $self->error_switch_sections();
}

sub success_switch_sections {
    my $self = shift;
    my $active_section=$self->active_section;
    $active_section->success() if $active_section;
}

sub error_switch_sections {
    my $self = shift;
    my $active_section=$self->active_section;
    $active_section->error() if $active_section;
    $self->is_switched_to_error(1);
    my $source=$self->source;
    $source->error() if $source->DOES('Logoved::Role::Stream::Out::Pipe');
}

1;
__END__
